@@ -1,11 +1,11 @@
-## ---- echo=TRUE, results='markup', message=FALSE-------------------------
+## ----metals data, echo=TRUE, results='markup', message=FALSE-------------
 library("qgcomp")
 library("knitr")
 library("ggplot2")
 data("metals", package="qgcomp")
 head(metals)
 
-## ---- results='markup', fig.show='hold', fig.height=5, fig.width=7.5, cache=FALSE----
+## ----linear model and runtime, results='markup', fig.show='hold', fig.height=5, fig.width=7.5, cache=FALSE----
 # we save the names of the mixture variables in the variable "Xnm"
 Xnm <- c(
     'arsenic','barium','cadmium','calcium','chromium','copper',
@@ -45,7 +45,7 @@ qc.fit
 
 
 
-## ---- results='markup', fig.show='hold', fig.height=5, fig.width=7.5, cache=FALSE----
+## ----logistic qgcomp, results='markup', fig.show='hold', fig.height=5, fig.width=7.5, cache=FALSE----
 
 qc.fit2 <- qgcomp.noboot(disease_state~., expnms=Xnm, 
           data = metals[,c(Xnm, 'disease_state')], family=binomial(), 
@@ -69,7 +69,7 @@ qcboot.fit2
 qcboot.fit2b
 
 
-## ---- results='markup', fig.show='hold', fig.height=5, fig.width=7.5, cache=FALSE----
+## ----adjusting for covariates, results='markup', fig.show='hold', fig.height=5, fig.width=7.5, cache=FALSE----
 
 qc.fit3 <- qgcomp.noboot(y ~ mage35 + arsenic + barium + cadmium + calcium + chloride + 
                            chromium + copper + iron + lead + magnesium + manganese + 
@@ -86,16 +86,17 @@ qcboot.fit3 <- qgcomp.boot(y ~ mage35 + arsenic + barium + cadmium + calcium + c
                          seed=125)
 qc.fit3
 plot(qcboot.fit3)
+plot(qcboot.fit3, pointwiseref = 3)
 
 
-## ---- results='markup', fig.show='hold', fig.height=5, fig.width=7.5, cache=FALSE----
+## ----non-linear non-hom intro, results='markup', fig.show='hold', fig.height=5, fig.width=7.5, cache=FALSE----
 
 qcboot.fit4 <- qgcomp(y~. + .^2,
                          expnms=Xnm,
                          metals[,c(Xnm, 'y')], family=gaussian(), q=4, B=10, seed=125)
 plot(qcboot.fit4)
 
-## ---- results='markup', fig.show='hold', fig.height=5, fig.width=7.5, cache=FALSE----
+## ----overall non-linearity, results='markup', fig.show='hold', fig.height=5, fig.width=7.5, cache=FALSE----
 
 qcboot.fit5 <- qgcomp(y~. + .^2,
                          expnms=Xnm,
@@ -103,7 +104,7 @@ qcboot.fit5 <- qgcomp(y~. + .^2,
                       B=10, rr=FALSE, seed=125)
 plot(qcboot.fit5)
 
-## ---- results='markup', fig.show='hold', fig.height=3, fig.width=7.5, cache=FALSE----
+## ----graphical non-linearity, results='markup', fig.show='hold', fig.height=3, fig.width=7.5, cache=FALSE----
 library(splines)
 # find all correlations > 0.6 (this is an arbitrary choice)
 cormat = cor(metals[,Xnm])
@@ -130,18 +131,28 @@ qc.fit6nonhom <- qgcomp.boot(y ~ bs(iron)*bs(lead) + bs(iron)*bs(cadmium) + bs(l
                          metals, family=gaussian(), q=8, B=10, degree=3)
 # it helps to place the plots on a common y-axis, which is easy due 
 #  to dependence of the qgcomp plotting functions on ggplot
-pl.fit6lin <- plot(qc.fit6lin, suppressprint = TRUE)
-pl.fit6nonlin <- plot(qc.fit6nonlin, suppressprint = TRUE)
-pl.fit6nonhom <- plot(qc.fit6nonhom, suppressprint = TRUE)
+pl.fit6lin <- plot(qc.fit6lin, suppressprint = TRUE, pointwiseref = 4)
+pl.fit6nonlin <- plot(qc.fit6nonlin, suppressprint = TRUE, pointwiseref = 4)
+pl.fit6nonhom <- plot(qc.fit6nonhom, suppressprint = TRUE, pointwiseref = 4)
 
 pl.fit6lin + coord_cartesian(ylim=c(-0.75, .75)) + 
   ggtitle("Linear fit: mixture of iron, lead, and cadmium")
 pl.fit6nonlin + coord_cartesian(ylim=c(-0.75, .75)) + 
-  ggtitle("Linear fit: mixture of iron, lead, and cadmium")
+  ggtitle("Non-linear fit: mixture of iron, lead, and cadmium")
 pl.fit6nonhom + coord_cartesian(ylim=c(-0.75, .75)) + 
   ggtitle("Non-linear, non-homogeneous fit: mixture of iron, lead, and cadmium")
 
-## ---- results='markup', fig.show='hold', fig.height=3, fig.width=7.5, cache=FALSE----
+## ----graphical caution, results='markup', fig.show='hold', fig.height=3, fig.width=7.5, cache=FALSE----
+qc.overfit <- qgcomp.boot(y ~ bs(iron) + bs(cadmium) + bs(lead) +
+                         mage35 + bs(arsenic) + bs(magnesium) + bs(manganese) + bs(mercury) + 
+                         bs(selenium) + bs(silver) + bs(sodium) + bs(zinc),
+                         expnms=Xnm,
+                         metals, family=gaussian(), q=8, B=10, degree=1)
+qc.overfit
+plot(qc.overfit, pointwiseref = 5)
+plot(qc.overfit, flexfit = FALSE, pointwiseref = 5)
+
+## ----non-linear examples, results='markup', fig.show='hold', fig.height=3, fig.width=7.5, cache=FALSE----
 
 # using indicator terms for each quantile
 qc.fit7a <- qgcomp.boot(y ~ factor(iron) + lead + cadmium + 
@@ -175,7 +186,7 @@ plot(qc.fit7c)
 
 
 
-## ---- results='markup', fig.show='hold', fig.height=5, fig.width=7.5, cache=FALSE----
+## ----splines, results='markup', fig.show='hold', fig.height=5, fig.width=7.5, cache=FALSE----
 AIC(qc.fit6lin$fit)
 AIC(qc.fit6nonlin$fit)
 AIC(qc.fit6nonhom$fit)
@@ -184,7 +195,7 @@ BIC(qc.fit6lin$fit)
 BIC(qc.fit6nonlin$fit)
 BIC(qc.fit6nonhom$fit)
 
-## ---- results='markup', fig.show='hold', fig.height=5, fig.width=7.5, cache=FALSE----
+## ----time-to-event, results='markup', fig.show='hold', fig.height=5, fig.width=7.5, cache=FALSE----
 # non-bootstrapped version estimates a marginal structural model for the 
 # confounder-conditional effect
 survival::coxph(survival::Surv(disease_time, disease_state) ~ iron + lead + cadmium + 
@@ -201,21 +212,21 @@ plot(qc.survfit1)
 #library(survival)
 qc.survfit2 <- qgcomp.cox.boot(Surv(disease_time, disease_state) ~ .,expnms=Xnm,
                          data=metals[,c(Xnm, 'disease_time', 'disease_state')], q=4, 
-                         B=10, MCsize=1000, parallel=TRUE)
+                         B=5, MCsize=1000, parallel=TRUE)
 qc.survfit2
 p2 = plot(qc.survfit2, suppressprint = TRUE)  
 p2 + labs(title="Linear log(hazard ratio), overall and exposure specific")
 
 qc.survfit3 <- qgcomp.cox.boot(Surv(disease_time, disease_state) ~ . + .^2,expnms=Xnm,
                          data=metals[,c(Xnm, 'disease_time', 'disease_state')], q=4, 
-                         B=10, MCsize=1000, parallel=TRUE)
+                         B=5, MCsize=1000, parallel=TRUE)
 qc.survfit3
 p3 = plot(qc.survfit3, suppressprint = TRUE) 
-p3 + labs(title="Non-linear log(hazard ratio), exposure specific")
+p3 + labs(title="Non-linear log(hazard ratio) overall, linear exposure specific ln-HR")
 
 qc.survfit4 <- qgcomp.cox.boot(Surv(disease_time, disease_state) ~ . + .^2,expnms=Xnm,
                          data=metals[,c(Xnm, 'disease_time', 'disease_state')], q=4, 
-                         B=10, MCsize=1000, parallel=TRUE, degree=2)
+                         B=5, MCsize=1000, parallel=TRUE, degree=2)
 qc.survfit4
 # examining the overall hazard ratio as a function of overall exposure
 hrs_q = exp(matrix(c(0,0,1,1,2,4,3,9), ncol=2, byrow=TRUE)%*%qc.survfit4$msmfit$coefficients)
